@@ -2,7 +2,7 @@
 #### based originally on Eva Cantoni's S-plus code "robGLM"
 
 if(getRversion() > "2.15.0" || as.numeric(R.Version()$`svn rev`) > 59233)
-    utils::globalVariables(c("residP", "residPS", "dmu.deta"), add=TRUE)
+    globalVariables(c("residP", "residPS", "dmu.deta"), add=TRUE)
 
 glmrobMqle <-
     function(X, y, weights = NULL, start = NULL, offset = NULL,
